package client

import (
	"encoding/json"
	"errors"

	"git.selly.red/Selly-Modules/natsio"
	"git.selly.red/Selly-Modules/natsio/model"
	"git.selly.red/Selly-Modules/natsio/subject"
)

// Bank ...
type Bank struct{}

// GetBank ...
func GetBank() Bank {
	return Bank{}
}

// GetBankInfo ...
func (s Bank) GetBankInfo(p []model.GetBankInfoRequest) ([]*model.ResponseBankInfo, error) {
	msg, err := natsio.GetServer().Request(subject.Bank.GetBankInfo, toBytes(p))
	if err != nil {
		return nil, err
	}

	var r struct {
		Data  []*model.ResponseBankInfo `json:"data"`
		Error string                    `json:"error"`
	}

	if err = json.Unmarshal(msg.Data, &r); err != nil {
		return nil, err
	}
	if r.Error != "" {
		return nil, errors.New(r.Error)
	}

	return r.Data, nil
}

func (s Bank) CheckBankAndBranchByID(p model.CheckBankAndBranchByIDRequest) bool {
	msg, err := natsio.GetServer().Request(subject.Bank.CheckBankAndBranchByID, toBytes(p))
	if err != nil {
		return false
	}

	var r struct {
		Data bool `json:"data"`
	}

	if err = json.Unmarshal(msg.Data, &r); err != nil {
		return false
	}
	return r.Data
}

// GetBankAndBranchDetail ...
func (s Bank) GetBankAndBranchDetail(p model.GetBankInfoRequest) (*model.ResponseBankInfo, error) {
	msg, err := natsio.GetServer().Request(subject.Bank.GetBankInfoDetail, toBytes(p))
	if err != nil {
		return nil, err
	}

	var r struct {
		Data  *model.ResponseBankInfo `json:"data"`
		Error string                  `json:"error"`
	}

	if err = json.Unmarshal(msg.Data, &r); err != nil {
		return nil, err
	}
	if r.Error != "" {
		return nil, errors.New(r.Error)
	}

	return r.Data, nil
}
